# --8<--8<--8<--8<--
#
# Copyright (C) 2015 Smithsonian Astrophysical Observatory
#
# This file is part of CXC::ACIS::ParameterBlock
#
# CXC::ACIS::ParameterBlock is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -->8-->8-->8-->8--

package CXC::ACIS::ParameterBlock;

use 5.10.1;

use strict;
use warnings;

use Carp;

our $VERSION = '0.02';

use CXC::ACIS::Command::FITS qw[ loadFITS ];
use CXC::ACIS::ParameterBlock::Types '-all';

use Moo;
use MooX::StrictConstructor;

has block => (
    is      => 'ro',
    isa     => Object,
    handles => [ qw(
          commandIdentifier
          commandLength
          commandName
          commandOpcode
          )
    ],

);

sub new_from_block {

    my $class = shift;
    my ( $block ) = @_;

    for ( $block->commandName ) {

	when( 'loadCcBlock' ) {

	    require CXC::ACIS::ParameterBlock::CC;

	    return CXC::ACIS::ParameterBlock::CC->new_from_block( $block );

	}

	when( 'loadTeBlock' ) {

	    require CXC::ACIS::ParameterBlock::TE;

	    return CXC::ACIS::ParameterBlock::TE->new_from_block( $block );

	}

	when( 'load2dBlock' ) {

	    require CXC::ACIS::ParameterBlock::Window2D;
	    return CXC::ACIS::ParameterBlock::Window2D->new_from_block( $block );

	}

	default {

	    croak( "unsupported parameter block: ", $block->commandName );

	}

    }



}


sub new_from_fits {

    my $class = shift;
    my $file = shift;

    my $block = Hash::AsObject->new( loadFITS( $file ) );

    return $class->new_from_block( $block );

}


1;


__END__

=head1 NAME

CXC::ACIS::ParameterBlock - Represent an ACIS parameter block as an object;


=head1 SYNOPSIS

    use CXC::ACIS::ParameterBlock;
    $pbk = CXC::ACIS::ParameterBlock->new_from_fits( $fits_file_name );



=head1 DESCRIPTION

This class is a base clas for represention of ACIS Parameter Blocks.
It is not only a base class, it currently provides the only useful means of
generating subclasses for the support parameter blocks.

It supports the following blocks:

=over

=item

C<loadCcBlock> (CC Mode)

=item

C<loadTeBlock> (TE Mode)

=item

C<load2dBlock> (2D Windows)

=back



=head1 INTERFACE

=head2 Constructors

=head3 new_from_fits

  $pbk = CXC::ACIS::ParameterBlock->new_from_fits( $fits_file_name );

Construct a new object from the given CXC pipeline generated FITS file.  It will
automatically determine which block is represented by the FITS file, and return
an object constructed from one of the following classes:

=over

=item

B<L<CXC::ACIS::ParameterBlock::TE>>

=item

B<L<CXC::ACIS::ParameterBlock::CC>>

=item

B<L<CXC::ACIS::ParameterBlock::Window2D>>

=back

=head2 Methods

The following methods are provided by this class; more are provided by
the subclasses.

=head3 commandIdentifier

=head3 commandLength

=head3 commandName

=head3 commandOpcode


=head1 BUGS AND LIMITATIONS

No bugs have been reported.


=head1 SEE ALSO

B<L<CXC::ACIS::ParameterBlock::TE>>,
B<L<CXC::ACIS::ParameterBlock::CC>>,
B<L<CXC::ACIS::ParameterBlock::Window2D>>.


=head1 LICENSE AND COPYRIGHT

Copyright (c) 2015 The Smithsonian Astrophysical Observatory

CXC::ACIS::ParameterBlock is free software: you can redistribute it
and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=head1 AUTHOR

Diab Jerius  E<lt>djerius@cfa.harvard.eduE<gt>
