# --8<--8<--8<--8<--
#
# Copyright (C) 2015 Smithsonian Astrophysical Observatory
#
# This file is part of CXC::ACIS::ParameterBlock
#
# CXC::ACIS::ParameterBlock is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -->8-->8-->8-->8--

package CXC::ACIS::ParameterBlock::FEP;

use 5.10.0;
use strict;
use warnings;

use String::Formatter named_stringf => {
    codes => {
        s => sub { $_ },
        d => sub { sprintf '%d', $_ },
    },
};


use Moo;
use MooX::StrictConstructor;

use CXC::ACIS::ParameterBlock::Types (
    qw[ CCDid FEPid Tuple PositiveOrZeroInt ] );

has ccdId => (
    is       => 'ro',
    isa      => CCDid,
    required => 1,
);

has fepId => (
    is       => 'ro',
    isa      => FEPid,
    required => 1,
);

has EventThreshold => (
    is       => 'ro',
    isa      => Tuple [ ( PositiveOrZeroInt ) x 4 ],
    required => 1,
);

has SplitThreshold => (
    is       => 'ro',
    isa      => Tuple [ ( PositiveOrZeroInt ) x 4 ],
    required => 1,
);

has biasAlgorithmId => (
    is       => 'ro',
    isa      => PositiveOrZeroInt,
    required => 1,
);

has ccdVideoResponse => (
    is       => 'ro',
    isa      => PositiveOrZeroInt,
    required => 1,
);

has VideoOffset => (
    is       => 'ro',
    isa      => Tuple [ ( PositiveOrZeroInt ) x 4 ],
    required => 1,
);


sub new_from_block {

    my $class = shift;

    return $class->new( $class->_attrs_from_block( @_ ) );
}


sub _attrs_from_block {

    my $class = shift;

    my ( $block, $fep, %fields ) = @_;

    my %attr;

    ## no critic (ProhibitAccessOfPrivateData)

    $attr{fepId} = $fep;
    $attr{ccdId} = $block->fepCcdSelect->[$fep];

    $fields{idx_by_fep} = [ 'biasAlgorithmId',
        'ccdVideoResponse', @{ $fields{idx_by_fep} // [] }, ];

    $attr{$_} = $block->$_->[$fep] for @{ $fields{idx_by_fep} };

    $fields{attr_by_fep} = [
        [ EventThreshold => 'fep%{fep}d%{attr}s' ],
        [ SplitThreshold => 'fep%{fep}d%{attr}s' ],
        [ VideoOffset    => 'fep%{fep}d%{attr}s' ],
        @{ $fields{attr_by_fep} // [] },
    ];


    for my $field ( @{ $fields{attr_by_fep} } ) {

        my ( $attr, $fmt ) = @$field;
        my $mth = named_stringf( $fmt, { attr => $attr, fep => $fep } );

        $attr{$attr} = $block->$mth;
    }

    return \%attr;
}


1;




__END__

=head1 NAME

CXC::ACIS::ParameterBlock::FEP - Represent an ACIS parameter block FEP entry as an object;


=head1 DESCRIPTION

This class is a base class for ACIS Parameter Block FEP entries. Use
one of its subclasses.


=head1 INTERFACE

=head2 Attributes

The following attributes are supported:

=head3 ccdId

=head3 fepId

=head3 EventThreshold

=head3 SplitThreshold

=head3 biasArg

=head3 biasCompressionSlotIndex

=head3 biasAlgorithmId

=head3 ccdVideoResponse

=head3 VideoOffset


=head1 BUGS AND LIMITATIONS

=for author to fill in:
    A list of known problems with the module, together with some
    indication Whether they are likely to be fixed in an upcoming
    release. Also a list of restrictions on the features the module
    does provide: data types that cannot be handled, performance issues
    and the circumstances in which they may arise, practical
    limitations on the size of data sets, special cases that are not
    (yet) handled, etc.

No bugs have been reported.


=head1 SEE ALSO

L<B<CXC::ACIS::ParameterBlock::FEP::CC>>, L<B<CXC::ACIS::ParameterBlock::FEP::TE>>


=head1 VERSION

Version 0.01

=head1 LICENSE AND COPYRIGHT

Copyright (c) 2015 The Smithsonian Astrophysical Observatory

CXC::ACIS::ParameterBlock is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=head1 AUTHOR

Diab Jerius  E<lt>djerius@cfa.harvard.eduE<gt>
