# --8<--8<--8<--8<--
#
# Copyright (C) 2015 Smithsonian Astrophysical Observatory
#
# This file is part of CXC::ACIS::ParameterBlock
#
# CXC::ACIS::ParameterBlock is free software: you can redistribute it
# and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -->8-->8-->8-->8--

package CXC::ACIS::ParameterBlock::Mode;

use 5.10.0;

use Moo;
use MooX::StrictConstructor;

use Type::Params qw[ validate ];
use CXC::ACIS::ParameterBlock::Types '-all';

extends 'CXC::ACIS::ParameterBlock';

has feps => (
    is       => 'ro',
    isa      => Map [ FEPid, InstanceOf ['CXC::ACIS::ParameterBlock::FEP'] ],
    required => 1,
);


has '+block' => (
    is      => 'ro',
    isa     => Object,
    handles => [ qw(
          bepPackingMode
          biasAlgorithmId
          ccdVideoResponse
          checksum
          deaLoadOverride
          eventAmplitudeRange
          fepCcdSelect
          fepLoadOverride
          fepMode
          gradeSelections
          ignoreBadColumnMap
          ignoreInitialFrames
          lowerEventAmplitude
          outputRegisterMode
          overclockPairsPerNode
          parameterBlockId
          rawCompressionSlotIndex
          recomputeBias
          trickleBias
          windowSlotIndex
          )
    ],

);

sub new_from_block {

    my $class = shift;
    my ( $block ) = @_;

    ## no critic (ProhibitAccessOfPrivateData)
    my @fep_ids = grep { $block->fepCcdSelect->[$_] != 10 }
      0 .. $#{ $block->fepCcdSelect };

    my %feps = map { $_ => $class->fep_from_block( $block, $_ ) } @fep_ids;

    return $class->new(
        feps  => \%feps,
        block => $block
    );

}

has fepIds => (
    is       => 'lazy',
    init_arg => undef,
    builder  => sub {
        [ map { $_->fepId } @{ $_[0]->feps } ];
    },

);

has ccdIds => (
    is       => 'lazy',
    init_arg => undef,
    builder  => sub {
        [ map { $_->ccdId } @{ $_[0]->feps } ];
    },

);

sub fep {

    my $self = shift;

    my ( $ccdId ) = validate( \@_, CCDid );

    my ( $fep ) = grep { $_->ccdId == $ccdId } @{ $self->feps };


}

1;

__END__

=head1 NAME

CXC::ACIS::ParameterBlock::Mode - Base class for representing a TE or CC Mode parameter block

=head1 DESCRIPTION

B<L<CXC::ACIS::ParameterBlock::Mode>> is a base class for TE and CC mode parameter blocks.

It is a subclass of B<L<CXC::ACIS::ParameterBlock>>.

This class isn't terribly useful on its own.  See B<L<CXC::ACIS::ParameterBlock::TE>>
and B<L<CXC::ACIS::ParameterBlock::CC>>.

=head1 INTERFACE

=head2 Methods

This class provides the following methods in addition to those of its
parent classes.

=head3 feps

Return an arrayref of B<L<CXC::ACIS::ParameterBlock::FEP>> objects.

=head3 fepIds

Return an arrayref containing the C<fepId>s of the associated FEPs

=head3 ccdIds

Return an arrayref containing the C<ccdId>s of the associated FEPs

=head2 Attributes

This class provides the following attributes in addition to those of its
parent classes.


=head3 bepPackingMode

=head3 biasAlgorithmId

=head3 ccdVideoResponse

=head3 checksum

=head3 deaLoadOverride

=head3 eventAmplitudeRange

=head3 fepCcdSelect

=head3 fepLoadOverride

=head3 fepMode

=head3 gradeSelections

=head3 ignoreBadColumnMap

=head3 ignoreInitialFrames

=head3 lowerEventAmplitude

=head3 outputRegisterMode

=head3 overclockPairsPerNode

=head3 parameterBlockId

=head3 rawCompressionSlotIndex

=head3 recomputeBias

=head3 trickleBias

=head3 windowSlotIndex



=head1 BUGS AND LIMITATIONS

No bugs have been reported.


=head1 SEE ALSO

B<L<CXC::ACIS::ParameterBlock>>, B<L<CXC::ACIS::ParameterBlock::FEP>>, B<L<CXC::ACIS::ParameterBlock::CC>>, B<L<CXC::ACIS::ParameterBlock::TE>>.

=head1 LICENSE AND COPYRIGHT

Copyright (c) 2015 The Smithsonian Astrophysical Observatory

CXC::ACIS::ParameterBlock is free software: you can redistribute it
and/or modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=head1 AUTHOR

Diab Jerius  E<lt>djerius@cfa.harvard.eduE<gt>
